%--------------------------------------------------------------------------
%------- Main code Chapter 2: Returns ------
%--------------------------------------------------------------------------
%---- This code is meant to help the reader going through the graphs and --
%---- estimates reported in Chapter 2 of the book                      --
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%------- Housekeeping and Load Data-----------------------------------------------------
%--------------------------------------------------------------------------
close all; clear all; clc; pause(0.01), randn('seed',sum(clock)), rand('seed',sum(clock)), warning off

addpath([pwd '\Input\']);
addpath([pwd '\Utilities\']);
addpath([pwd '\Output\']);
[data,textdata,raw] = xlsread('ie_data.xls','Data');
%% Time Series
US.P   = data(:,2);
US.D = data(:,3)./12;
US.CPI      = data(:,5);
US.BondYTM  = data(:,7);
US.DPratio      = US.D./US.P;
US.LogDPratio   = log(US.DPratio);
US.LogP     = log(US.P);
US.LogD     = log(US.D);
US.DivGrowth    =NaN(size(US.P));
US.DivGrowth(2:end,:)    = diff(US.LogD);
US.LogCPI       = log(US.CPI);
US.Returns.Monthly_lw    =NaN(size(US.P));
US.Returns.Monthly_lc    =NaN(size(US.P));
US.Returns.Monthly_ll    =NaN(size(US.P));
US.Returns.Monthly_w    =NaN(size(US.P));
US.Returns.Monthly_c    =NaN(size(US.P));
US.Returns.Monthly_bond =NaN(size(US.P));

US.Infl    =NaN(size(US.P));
US.Infl(2:end,:)             = diff(US.LogCPI);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% linearization parameters
US.DP_bar = mean(US.DPratio(2:end-3));
rho=(1/US.DP_bar)/(1+1/US.DP_bar);
k=-log(1-rho)-rho*log((1/(1-rho))-1);

% log monthly stock returns
US.Returns.Monthly_lw = log(US.P./lag(US.P,1,NaN));
US.Returns.Monthly_lc = log((US.P+US.D)./lag(US.P,1,NaN));
US.Returns.Monthly_ll=k-rho*US.LogDPratio+(US.LogD-lag(US.LogD,1,NaN))+lag(US.LogDPratio,1,NaN);
% linear monthly stock returns
US.Returns.Monthly_w = (US.P./lag(US.P,1,NaN))-1;
US.Returns.Monthly_c =((US.P+US.D)./lag(US.P,1,NaN))-1;
%Log monthly returns on Long term bonds
US.BondYTM_l   = (log(1+(data(:,7))/(100*12)));
US.dur = 12*((1-(1+(data(:,7)/(100))).^(-10)))./(1-(1+(data(:,7)/(100))).^(-1));
US.Returns.Monthly_bond =(lag(US.dur,1,NaN).*lag(US.BondYTM_l,1,NaN)-(lag(US.dur,1,NaN)-1).*US.BondYTM_l);

%--------------------------------------------------------------------------
%-------- Creates returns at different frequencies ------------------------
%--------------------------------------------------------------------------
% Calculate log returns
t = [1:size((US.P))];
US.Returns.Yearly_lc   =NaN(size(US.P));
for i=12:size(t,2)
US.Returns.Yearly_lc(i,:) = sum(US.Returns.Monthly_lc(t(i)-11:t(i)));
end
US.Returns.TenYears_lc= NaN(size(US.P));
for i=121:size(t,2)
US.Returns.TenYears_lc(i,:) = sum(US.Returns.Monthly_lc(t(i)-120:t(i)));
end
US.Returns.TenYears_lca=US.Returns.TenYears_lc/10;

% exact returns
US.Returns.Yearly_c   =NaN(size(US.P));
for i=12:size(t,2)
US.Returns.Yearly_c(i,:) = prod(1+US.Returns.Monthly_c(t(i)-11:t(i)))-1;
end
US.Returns.TenYears_c   =NaN(size(US.P));
for i=121:size(t,2)
US.Returns.TenYears_c(i,:) = prod(1+US.Returns.Monthly_c(t(i)-120:t(i)))-1;
end
US.Returns.TenYears_ca=((1+US.Returns.TenYears_c).^0.1)-1;
%--------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Defining Sample 
date=datenum(textdata(8:end,1),'dd/mm/yyyy');%Convert dates into a serial number
s_start1 = '15/01/1948';
s_end1   = '15/11/2013';
date_find1=datenum([s_start1; s_end1],'dd/mm/yyyy'); % Lookup row number
ss1=datefind(date_find1(1,1),date); % Sample Start (First Raw)
se1=datefind(date_find1(2,1),date); % Sample End (Last Raw)
ObsSample = se1-ss1+1; % Number of observations within Sample
%% Analyzing Returns and Historical Performance 
Port_w=NaN(size(US.P));
Port_c=NaN(size(US.P));
Port_w(ss1,1)=1;
Port_c(ss1,1)=1;
Port_b(ss1,1)=1;
for i=ss1+1:se1
  Port_w(i,1)=Port_w(i-1,1)*(1+US.Returns.Monthly_w(i,1));  
  Port_c(i,1)=Port_c(i-1,1)*(1+US.Returns.Monthly_c(i,1));
  Port_b(i,1)=Port_b(i-1,1)*(1+US.Returns.Monthly_bond(i,1));
end
Port = [Port_w(ss1:se1) Port_c(ss1:se1) Port_b(ss1:se1)];
% Plot performances  in figure(1)
run Figure001   
% plot of returns and log returns
run Figure002
run Figure003
run Figure004

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Monte-Carlo and Bootstrap
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s_start1 = '15/01/1880';
s_end1   = '15/09/2013';
date_find1=datenum([s_start1; s_end1],'dd/mm/yyyy'); % Lookup row number
ss1=datefind(date_find1(1,1),date); % Sample Start (First Raw)
se1=datefind(date_find1(2,1),date); % Sample End (Last Raw)
obs = se1-ss1+1; % Number of observations within Sample
alpha = mean(US.Returns.Monthly_c(ss1:se1));
vol=std(US.Returns.Monthly_c(ss1:se1));
%----------------------------------------------------------------------
% A MONTE CARLO EXPERIMENT
%----------------------------------------------------------------------
% Generate error terms :
clear x y u1  
reps=1000;
x=NaN(obs,reps);
y=NaN(obs,reps);
u=NaN(obs,reps);

% Initializing the series:

y(1,:)=1;
x(1,:)=0;
% Artificial series:
u=vol.*normrnd(0,1,obs,reps);
 for j=1:reps  
for i=2:obs
x(i,j)=alpha+u(i,j); 
y(i,j)=(1+x(i,j))*y(i-1,j);
end
 end
 %Observed Series
 Cumret(ss1,1)=1;
for i=ss1+1:se1
    Cumret(i,1)=Cumret(i-1,1)*(1+US.Returns.Monthly_c(i,1));
end
SIM_MC_Cumret=prctile(y',[ 25 50 75 ]);
SIM_MC_ret=prctile(x',[ 25 50 75 ]);
% Plot performances  
run Figure005

%----------------------------------------------------------------------
% A BOOTSTRAP EXPERIMENT
%----------------------------------------------------------------------
% Generate error terms :
clear x1 y1 u1  
reps=1000;
x1=NaN(obs,reps);
y1=NaN(obs,reps);
u_boot=NaN(obs,reps);

% Initializing the series:
y1(1,:)=1;
x1(1,:)=0;
% Preparing Residuals for bootstrap:
u1=US.Returns.Monthly_c(ss1:se1)-mean(US.Returns.Monthly_c(ss1:se1));

%bootstrap
 for j=1:reps  
     u_boot(:,j)=randsample(u1,obs,'true');
for i=2:obs
x1(i,j)=alpha+u_boot(i,j); 
y1(i,j)=(1+x1(i,j))*y1(i-1,j);
end
 end
  
SIM_BS_Cumret=prctile(y1',[ 25 50 75 ]);
SIM_BS_ret=prctile(x1',[ 25 50 75 ]);
% Plot performances 
run Figure006

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 10-year investment performance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s_start1 = '15/01/1890';
s_end1   = '15/11/2013';
date_find1=datenum([s_start1; s_end1],'dd/mm/yyyy'); % Lookup row number
ss1=datefind(date_find1(1,1),date); % Sample Start (First Raw)
se1=datefind(date_find1(2,1),date); % Sample End (Last Raw)
ObsSample = se1-ss1+1;
Assets = [US.Returns.TenYears_lca(ss1:se1) 12*US.BondYTM_l(ss1-120:se1-120)];
Assets1 = [US.Returns.Monthly_lc(ss1:se1) US.Returns.Monthly_bond(ss1:se1)];
% Plot performances  
run Figure007
run Figure008

